/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Cherokee
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "common.h"

#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>
#include <string.h>

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "log.h"


int
hexit (char c)
{
	if ( c >= '0' && c <= '9' )
		return c - '0';
	if ( c >= 'a' && c <= 'f' )
		return c - 'a' + 10;
	if ( c >= 'A' && c <= 'F' )
		return c - 'A' + 10;

	/* shouldn't happen, we're guarded by isxdigit() */
	return 0;           
}


void 
PRINT_ERROR (const char *format, ...)
{
	va_list arg_list;
	CHEROKEE_TEMP(tmp, 2048);
	
	va_start(arg_list, format);
	vsnprintf (tmp, tmp_size, format, arg_list);
	va_end(arg_list);

	fprintf (stderr, "%s", tmp);
	syslog (LOG_ERR, tmp);
}


void 
PRINT_DEBUG (const char *format, ...)
{
	va_list arg_list;
	
	va_start(arg_list, format);
	vfprintf(stdout, format, arg_list);
	va_end(arg_list);

	fflush(stdout); 
}


/* This function is licenced under:
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  
 * All rights reserved.
 */
char *
apr_strfsize(size_t size, char *buf)
{
    const char ord[] = "KMGTPE";
    const char *o = ord;
    int remain;

    if (size < 0) {
        return strcpy(buf, "  - ");
    }
    if (size < 973) {
        sprintf(buf, "%3d ", (int) size);
        return buf;
    }
    do {
        remain = (int)(size & 1023);
        size >>= 10;
        if (size >= 973) {
            ++o;
            continue;
        }
        if (size < 9 || (size == 9 && remain < 973)) {
            if ((remain = ((remain * 5) + 256) / 512) >= 10)
                ++size, remain = 0;
            sprintf(buf, "%d.%d%c", (int) size, remain, *o);
            return buf;
        }
        if (remain >= 512)
            ++size;
        sprintf(buf, "%3d%c", (int) size, *o);
        return buf;
    } while (1);
}



char *
min_string_not_null (char *s1, char *s2)
{
	if ((s1 == NULL) && 
	    (s2 == NULL)) return NULL;

	if ((s1 != NULL) && 
	    (s2 == NULL)) return s1;

	if ((s2 != NULL) && 
	    (s1 == NULL)) return s2;
	
	return (s1<s2) ? s1 : s2;
}


ret_t 
get_system_fd_num_limit (int *limit)
{
	int nfiles;
	struct rlimit rl;

	 /* Figure out how many fd's we can have. 
	  */
	nfiles = getdtablesize();

#ifdef RLIMIT_NOFILE
	 /* If we have getrlimit(), use that, and attempt to raise the limit. 
	  */
	if (getrlimit (RLIMIT_NOFILE, &rl) == 0)
	{
		nfiles = rl.rlim_cur;
		
		if (rl.rlim_max == RLIM_INFINITY) {
			rl.rlim_cur = 8192; /* Arbitrary */
		} else if (rl.rlim_max > rl.rlim_cur) {
			rl.rlim_cur = rl.rlim_max;
		}
		
		if (setrlimit( RLIMIT_NOFILE, &rl ) == 0) {
			nfiles = rl.rlim_cur;
		}
	}
#endif
	   
#ifdef USE_SELECT
	/* If we use select(), then we must limit ourselves to FD_SETSIZE. 
	 */
	if (nfiles > FD_SETSIZE) {
		nfiles = FD_SETSIZE;
	}
#endif

	*limit = nfiles;
	return ret_ok;
}

