/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Cherokee
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __CHEROKEE_CONNECTION_HANDLER_CGI_H__
#define __CHEROKEE_CONNECTION_HANDLER_CGI_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "common.h"
#include "buffer.h"
#include "handler.h"


typedef struct {
	cherokee_handler_t handler;

	int pipeInput;   /* read from the CGI */
	int pipeOutput;  /* write to the CGI */

	int pid;         /* CGI pid */

	int post_data_sent;
	
	cherokee_buffer_t *pathInfo;
	cherokee_buffer_t *fileName;
	cherokee_buffer_t *parameter; 
	cherokee_buffer_t *data; 

} cherokee_handler_cgi_t;

#define CGIHANDLER(x)        ((cherokee_handler_cgi_t *)(x))
#define CGIHANDLER_FILE(x)  (((cherokee_handler_cgi_t *)(x))->fileName)
#define CGIHANDLER_PARAM(x) (((cherokee_handler_cgi_t *)(x))->parameter)

/* Library init function
 */
void cgi_init ();


ret_t cherokee_handler_cgi_new   (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties);

/* virtual methods implementation
 */
ret_t cherokee_handler_cgi_init        (cherokee_handler_cgi_t *hdl);
ret_t cherokee_handler_cgi_free        (cherokee_handler_cgi_t *hdl);
ret_t cherokee_handler_cgi_step        (cherokee_handler_cgi_t *hdl, cherokee_buffer_t *buffer);
ret_t cherokee_handler_cgi_add_headers (cherokee_handler_cgi_t *hdl, cherokee_buffer_t *buffer);

#endif /* __CHEROKEE_CONNECTION_HANDLER_CGI_H__ */
