%{
/* Cherokee
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "read_config_grammar.h"
#include "handler_table_entry.h"

#include <stdio.h>
#include <string.h>
%}

%option yylineno
%option nounput

ws              [ \t]+
integer         [-+]?[0-9]+
comment         #.*$
fpath           \/[^\"\{\}\n ]*
qstring         \"[^\"\{\}\n]*\"
http            http\:\/\/[a-zA-Z0-9\$\:\-_\.\&\=\;\/\?]*
id              [_\-\.a-zA-Z][\.\-_a-zA-Z0-9]*
ip4_end         [0-9]*[0-9]+
ip4_part        [0-2]*{ip4_end}
ip4_dir         {ip4_part}\.{ip4_part}\.{ip4_part}\.{ip4_part}
ip4_mask        [\/\\]({ip4_dir}|[0-9]{1,2})
ip4             {ip4_dir}{ip4_mask}{0,1}

hex_255         [0-9a-fA-F]{1,4}
ip6_part        {hex_255}":"?
ip6_form1       {ip6_part}{0,16}"::"{ip6_part}{0,16}
ip6_form2       ({hex_255}":"){16}{hex_255}
ip6_addr        {ip6_form1}|{ip6_form2}
ip6_mask        [\/\\]("1"?[0-9]{1,2}|{ip6_addr})
ip6             {ip6_addr}{ip6_mask}?

ip              {ip4}|{ip6}
nl              [\n\r]

%%

"Handler"               { return T_HANDLER; }
"Port"                  { return T_PORT; }
"DocumentRoot"          { return T_DOCUMENT_ROOT; }
"Log"                   { return T_LOG; }
"Timeout"               { return T_TIMEOUT; }
"MimeFile"              { return T_MIME_FILE; }
"Directory"             { return T_DIRECTORY; }
"KeepAlive"             { return T_KEEPALIVE; }
"MaxKeepAliveRequests"  { return T_KEEPALIVE_MAX_REQUESTS; }
"ErrorDocument"         { return T_ERROR_DOCUMENT; }
"User"                  { return T_USER; }
"Group"                 { return T_GROUP; }
"Server"                { return T_SERVER; }
"UserDir"               { return T_USERDIR; }
"FileDir"               { return T_FILEDIR; }
"PidFile"               { return T_PIDFILE; }
"Listen"                { return T_LISTEN; }
"Chroot"                { return T_CHROOT; }
"HideVersion"           { return T_HIDE_VERSION; }
"HideServerName"        { return T_HIDE_SERVER_NAME; }
"Encoder"               { return T_ENCODER; }
"ThreadNumber"          { return T_THREAD_NUM; }
"DirectoryIndex"        { return T_DIRECTORYINDEX; }
"HeaderFile"            { return T_HEADER_FILE; }
"IPv6"                  { return T_IPV6; }
"Icons"                 { return T_ICONS; }
"Auth"                  { return T_AUTH; }
"Name"                  { return T_NAME; }
"Method"                { return T_METHOD; }
"PasswdFile"            { return T_PASSWDFILE; }
"Show"                  { return T_SHOW; }
"url"                   { return T_URL; }
"bgcolor"               { return T_BGCOLOR; }
"text"                  { return T_TEXT; }
"link"                  { return T_LINK; }
"vlink"                 { return T_VLINK; }
"alink"                 { return T_ALINK; }
"background"            { return T_BACKGROUND; }
"Allow"                 { return T_ALLOW; }
"Deny"                  { return T_DENY; }
"From"                  { return T_FROM; }
"socket"                { return T_SOCKET;}
"SSLCertificateFile"    { return T_SSL_CERT_FILE; }
"SSLCertificateKeyFile" { return T_SSL_CERT_KEY_FILE; }
"SSLCAListFile"         { return T_SSL_CA_LIST_FILE; }
"On"                    { yylval.number = 1; return T_NUMBER; }
"Off"                   { yylval.number = 0; return T_NUMBER; }

{http}           { yylval.string = strdup(yytext); return T_HTTP_URL; }
{id}             { yylval.string = strdup(yytext); return T_ID; }
{ip}             { yylval.string = strdup(yytext); return T_IP; }
{fpath}          { yylval.string = strdup(yytext); return T_FULLDIR; }
{qstring}        { int len = strlen(yytext);
                   yylval.string = (char *) malloc (len-1);
                   memcpy (yylval.string, yytext+1, len-2);
                   yylval.string[len-2]='\0';
                   return T_QSTRING; 
                 }

{integer}        { yylval.number = atoi(yytext); return T_NUMBER; }

{comment}        ;
{nl}             ;
{ws}             ;

<<EOF>>          { yyterminate(); }

.		       { return yytext[0]; }

%%

int 
yywrap()
{
       return 1;
}

